/*
    ExoMessage.cpp
*/

#include "ExoMessage.hpp"

namespace foo {

ExoMsg::ExoMsg (
        std::vector<double> && inputNumbers,
        std::string && inputText)
    noexcept
    :
    numbers (inputNumbers),
    text (inputText)
{}

ExoMsg::ExoMsg (
        const ExoMsg & other)
    :
    numbers (other.numbers),
    text (other.text)
{}

ExoMsg::~ExoMsg ()
{
}

void ExoMsg::Deleter::operator() (ExoMsg * p) noexcept
{
    delete p;
}

ExoMsg & ExoMsg::operator= (
        ExoMsg && other) noexcept
{
    numbers = std::move (other.numbers);
    text = std::move (other.text);
    return *this;
}

ExoMsg & ExoMsg::operator= (
        const ExoMsg & other)
{
    ExoMsg tmp (other);
    swap (tmp);
    return *this;
}

void ExoMsg::swap (ExoMsg & other) noexcept
{
    using std::swap;
    swap (numbers, other.numbers);
    swap (text, other.text);
}

ExoMsg::Pointer ExoMsg::create (
        CNumPtrPair inputNumbers,
        const char * inputText)
{
    std::vector<double> numbers (inputNumbers.first,
                                 inputNumbers.second);
    std::string text (inputText);
    
    return Pointer (
                new ExoMsg (std::move(numbers),
                            std::move(text)));
}

ExoMsg::Pointer ExoMsg::create (
        CNumPtrPair inputNumbers)
{
    return create (inputNumbers, "");
}

ExoMsg::Pointer ExoMsg::create (
        const char * inputText)
{
    return create (CNumPtrPair (nullptr, nullptr),
                   inputText);
}

ExoMsg::Pointer ExoMsg::clone () const
{
    std::vector<double> numsCopy (numbers);
    std::string textCopy (text);
    return Pointer (
                new ExoMsg (std::move(numsCopy),
                            std::move(textCopy)));
}

ExoMsg::NumPtrPair ExoMsg::getNumbers () noexcept
{
    double * first = numbers.data ();
    double * afterLast = first + numbers.size ();
    return NumPtrPair (first, afterLast);
}

void ExoMsg::appendNumber (double d)
{
    numbers.push_back (d);
}

void ExoMsg::clearNumbers () noexcept
{
    numbers.clear ();
}

const char * ExoMsg::getText () const noexcept
{
    return text.c_str ();
}

void ExoMsg::appendText (const char * inputText)
{
    text += inputText;
}

void ExoMsg::clearText () noexcept
{
    text.clear ();
}

} // namespace foo
