/*
    ExoMessage.hpp
*/

#ifndef _FOO_EXO_MESSAGE_HPP_
#define _FOO_EXO_MESSAGE_HPP_

#include "ApiMacros.h"  // FOO_API
#include <string>
#include <vector>
#include <utility>      // pair
#include <memory>       // unique_ptr

namespace foo {

class FOO_API ExoMsg
{
private:

    std::vector<double> numbers;
    std::string text;

    ExoMsg (std::vector<double> &&, std::string &&) noexcept;
    ExoMsg (const ExoMsg &);
    ~ExoMsg ();

    struct Deleter
    {
        void operator() (ExoMsg *) noexcept;
    };

public:

    ExoMsg & operator= (ExoMsg &&) noexcept;
    ExoMsg & operator= (const ExoMsg &);
    void swap (ExoMsg &) noexcept;

    typedef std::unique_ptr<ExoMsg,Deleter> Pointer;
    typedef std::pair<double*,double*> NumPtrPair;
    typedef std::pair<const double*,const double*> CNumPtrPair;
    
    static Pointer create (CNumPtrPair, const char *);
    static Pointer create (CNumPtrPair);
    static Pointer create (const char *);
    Pointer clone () const;

    NumPtrPair getNumbers () noexcept;
    void appendNumber (double);
    void clearNumbers () noexcept;
    
    const char * getText () const noexcept;
    void appendText (const char *);
    void clearText () noexcept;
};

inline void swap (ExoMsg & a, ExoMsg & b) noexcept
{
    a.swap (b);
}

} // namespace foo

#endif // _FOO_EXO_MESSAGE_HPP_
