/*
    PImplMessage.cpp
*/

#include "PImplMessage.hpp"
#include <vector>
#include <string>

namespace foo {

struct PImplMsg::Impl
{
    std::vector<double> numbers;
    std::string text;
};

void PImplMsg::Deleter::operator() (
        PImplMsg::Impl * p) noexcept
{
    delete p;
}

PImplMsg::PImplMsg (PImplMsg && other) noexcept
    :
    pImpl (std::move(other.pImpl))
{
}

PImplMsg::PImplMsg (const PImplMsg & other)
    :
    pImpl (new PImplMsg::Impl)
{
    if (other.pImpl)
    {
        pImpl->numbers = other.pImpl->numbers;
        pImpl->text = other.pImpl->text;
    }
}

PImplMsg & PImplMsg::operator= (
        PImplMsg && other) noexcept
{
    pImpl = std::move (other.pImpl);
    return *this;
}

PImplMsg & PImplMsg::operator= (
        const PImplMsg & other)
{
    PImplMsg tmp (other);
    swap (tmp);
    return *this;
}

void PImplMsg::swap (PImplMsg & other) noexcept
{
    using std::swap;
    swap (pImpl, other.pImpl);
}

PImplMsg::PImplMsg (
        CNumPtrPair inputNumbers,
        const char * inputText)
    :
    pImpl (new PImplMsg::Impl)
{
    pImpl->numbers = std::vector<double> (
                                 inputNumbers.first,
                                 inputNumbers.second);
    pImpl->text = inputText;
}

PImplMsg::PImplMsg (CNumPtrPair inputNumbers)
    :
    pImpl (new PImplMsg::Impl)
{
    pImpl->numbers = std::vector<double> (
                                 inputNumbers.first,
                                 inputNumbers.second);
}

PImplMsg::PImplMsg (const char * inputText)
    :
    pImpl (new PImplMsg::Impl)
{
    pImpl->text = inputText;
}

PImplMsg::NumPtrPair PImplMsg::getNumbers () noexcept
{
    if ( ! pImpl)
    {
        static double dummy; // (Non-static should be OK too)
        return NumPtrPair (&dummy, &dummy);
    }
        
    double * first     = pImpl->numbers.data ();
    double * afterLast = first +
                         pImpl->numbers.size ();
    return NumPtrPair (first, afterLast);
}

void PImplMsg::appendNumber (double d)
{
    if ( ! pImpl)
        pImpl.reset (new Impl);
        
    pImpl->numbers.push_back (d);
}

void PImplMsg::clearNumbers () noexcept
{
    if (pImpl)
        pImpl->numbers.clear ();
}

const char * PImplMsg::getText () const noexcept
{
    return pImpl ? pImpl->text.c_str () : "";
}

void PImplMsg::appendText (const char * inputText)
{
    if ( ! pImpl)
        pImpl.reset (new Impl);
        
    pImpl->text += inputText;
}

void PImplMsg::clearText () noexcept
{
    if (pImpl)
        pImpl->text.clear ();
}

} // namespace foo
