/*
    PImplMessage.hpp
*/

#ifndef _FOO_PIMPL_MESSAGE_HPP_
#define _FOO_PIMPL_MESSAGE_HPP_

#include "ApiMacros.h"  // FOO_API
#include <utility>      // pair
#include <memory>       // unique_ptr

namespace foo {

class FOO_API PImplMsg
{
private:

    struct Impl;

    struct Deleter
    {
        void operator() (Impl *) noexcept;
    };

    std::unique_ptr<Impl,Deleter> pImpl;

public:

    PImplMsg () noexcept {}
    PImplMsg (PImplMsg &&) noexcept;
    PImplMsg (const PImplMsg &);
    PImplMsg & operator= (PImplMsg &&) noexcept;
    PImplMsg & operator= (const PImplMsg &);
    void swap (PImplMsg &) noexcept;

    typedef std::pair<double*,double*> NumPtrPair;
    typedef std::pair<const double*,const double*> CNumPtrPair;
    
    PImplMsg (CNumPtrPair, const char *);
    explicit PImplMsg (CNumPtrPair);
    explicit PImplMsg (const char *);

    NumPtrPair getNumbers () noexcept;
    void appendNumber (double);
    void clearNumbers () noexcept;
    
    const char * getText () const noexcept;
    void appendText (const char *);
    void clearText () noexcept;
};

inline void swap (PImplMsg & a, PImplMsg & b) noexcept
{
    a.swap (b);
}

} // namespace foo

#endif // _FOO_PIMPL_MESSAGE_HPP_
